unit bassplayer;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, audiofile, bass, Dialogs, fileutil, lcltype;

type
 TBassPlayer = class
  private
   fMainStream: DWord;
   fMainVolume: Single;
   fMainAudioFile: TAudioFile;
   fOnPlay    : TNotifyEvent;
   fOnPause   : TNotifyEvent;
   fOnResume  : TNotifyEvent;
   fOnStop    : TNotifyEvent;
   fOnChange  : TNotifyEvent;
   fFilter           : string;
   fFadeIn           : DWord;
   fFadeOut          : DWord;
   fValidExtensions  : TStringList;
   function BP_CreateStream(aFilename: string): DWord;
   procedure SetVolume(Value: Single);
   function GetTime: Double;
   function GetProgress: Double;
   procedure SetProgress(Value: Double);
   procedure SetPosition(Value: Longword);
   function GetBassStatus: DWord;
   function GetPlaylistTitel: string;

  public
   property Volume:     Single read fMainVolume write SetVolume;
   property Time:       Double read GetTime;
   property Progress:   Double read GetProgress write SetProgress;
   property BassStatus: DWord  read GetBassStatus;
   property FadeIn    : DWord  read fFadeIn     write fFadeIn;
   property FadeOut   : DWord  read fFadeOut    write fFadeOut;
   property OnPlay    : TNotifyEvent read fOnPlay     write fOnPlay    ;
   property OnPause   : TNotifyEvent read fOnPause    write fOnPause   ;
   property OnResume  : TNotifyEvent read fOnResume   write fOnResume  ;
   property OnStop    : TNotifyEvent read fOnStop     write fOnStop    ;
   property OnChange  : TNotifyEvent read fOnChange   write fOnChange  ;
   property PlaylistTitel: string read GetPlaylistTitel;
   property Filter : string read fFilter;

   constructor Create;
   destructor Destroy; override;
   procedure InitBassEngine(HND: HWND);
   procedure Play(aAudioFile: TAudioFile; slide:boolean);
   procedure Pause;
   procedure Stop;
   procedure Resume;
   procedure StopAndFree(slide:boolean);
   procedure InitPlugins(PathToDlls:string);
   function  IsPlayableFile(aFilename:string):boolean;
   function  GetChannelLevel:Cardinal;
   procedure SetPan(Value:float);
   procedure Loop(bol:boolean);
 end;

implementation

function Explode(const Separator, S: String): TStringList;
var
  SepLen: Integer;
  F, P: PChar;
  tmpstr: String;
begin
  result := TStringList.Create;
  if (S = '') then Exit;
  if Separator = '' then
  begin
    Result.Add(S);
    Exit;
  end;
  SepLen := Length(Separator);
  P := PChar(S);
  while P^ <> #0 do
  begin
    F := P;
    P := StrPos(P, PChar(Separator));
    if (P = nil) then P := StrEnd(F);
    SetString(tmpstr, F, P - F);
    Result.Add(tmpstr);
    if P^ <> #0 then Inc(P, SepLen);
  end;
end;

constructor TBassPlayer.Create;
begin
  inherited create;
  fMainAudioFile := TAudioFile.Create;
end;

destructor TBassPlayer.Destroy;
begin
  fMainAudioFile.Free;
  BASS_Free;
  inherited Destroy;
end;

procedure TBassPlayer.InitBassEngine(HND:HWND);
begin
 if (HIWORD(BASS_GetVersion) <> BASSVERSION) then
  MessageDLG('Bass 2.4 nicht gefunden', mtError, [mbOk], 0);
 BASS_Init(-1,44100,0,HND,nil);
end;

procedure TBassPlayer.Play(aAudioFile:TAudioFile; slide:boolean);
begin
 if aAudioFile <> nil then
 begin
   fMainAudioFile.Assign(aAudioFile);
   StopAndFree(slide);
   fMainStream := BP_CreateStream(fMainAudioFile.Pfad);
   if slide then
    BASS_ChannelSetAttribute(fMainStream, BASS_ATTRIB_VOL, 0)
   else
    BASS_ChannelSetAttribute(fMainStream, BASS_ATTRIB_VOL, fMainVolume);
   BASS_ChannelPlay(fMainStream, true);
   if slide then
    BASS_ChannelSlideAttribute(fMainStream, BASS_ATTRIB_VOL, fMainVolume, fFadeIn);
 end;
 if assigned(fOnPlay) then fOnPlay(Self);
end;

procedure TBassPlayer.StopAndFree(slide:boolean);
begin
 if slide then
 begin
  BASS_ChannelFlags(fMainStream, BASS_STREAM_AUTOFREE, BASS_STREAM_AUTOFREE);
  BASS_ChannelSlideAttribute(fMainStream, BASS_ATTRIB_VOL, 0, fFadeOut);
 end else begin
  BASS_ChannelStop(fMainStream);
  BASS_StreamFree(fMainStream);
 end;
 fMainStream := 0;
end;

function TBassPlayer.BP_CreateStream(aFilename:string):DWORD;
var flags:DWORD;
begin
 if (AnsiLowerCase(ExtractFileExt(aFilename)) = '.mp3') then
  flags := BASS_STREAM_PRESCAN
 else
  flags := 0;
 result := BASS_StreamCreateFile(false, PChar(aFileName), 0, 0, flags);
end;

procedure TBassPlayer.Pause;
begin
 BASS_ChannelPause(fMainStream);
 if assigned(fOnPause) then fOnPause(Self);
end;

procedure TBassPlayer.Stop;
begin
 BASS_ChannelStop(fMainStream);
 if assigned(fOnStop) then fOnStop(Self);
end;

procedure TBassPlayer.Resume;
begin
 BASS_ChannelPlay(fMainStream, false);
 if assigned(fOnResume) then fOnResume(Self);
end;

procedure TBassPlayer.SetVolume(Value: Single);
begin
  if Value < 0 then Value := 0;
  if Value > 1 then Value := 1;
  fMainVolume := Value;
  BASS_ChannelSetAttribute(fMainStream, BASS_ATTRIB_VOL, fMainVolume);
  fMainVolume := Value;
end;

function TBassPlayer.GetTime: Double;
begin
  if (fMainStream <> 0) then
    result := BASS_ChannelBytes2Seconds(fMainStream,
                    BASS_ChannelGetPosition(fMainStream, BASS_POS_BYTE))
  else
    result := 0;
end;

function TBassPlayer.GetProgress: Double;
begin
  if (fMainStream <> 0) then
    result := BASS_ChannelGetPosition(fMainStream, BASS_POS_BYTE) /
                    BASS_ChannelGetLength(fMainStream, BASS_POS_BYTE)
  else
    result := 0;
end;

procedure TBassPlayer.SetPosition(Value: Longword);
begin
  BASS_ChannelSetPosition(fMainStream, Value, BASS_POS_BYTE);
end;

procedure TBassPlayer.SetProgress(Value: Double);
begin
  if Value < 0 then Value := 0;
  if Value > 1 then Value := 1;
  SetPosition(Round(BASS_ChannelGetLength(fMainStream, BASS_POS_BYTE) * Value));
end;

function TBassPlayer.GetBassStatus: DWord;
begin
  result := BASS_ChannelIsActive(fMainStream);
end;

function TBassPlayer.GetPlaylistTitel: string;
begin
  if assigned(fMainAudioFile) then
    result := fMainAudioFile.PlaylistTitel
  else
    result := '-';
end;

procedure TBassPlayer.InitPlugins(PathToDlls:string);
var dlllist:TStringList;
    Plug  :DWORD;
    Info  :PBass_PluginInfo;
    a,i,ii   :integer;
    tmpext:TStringlist;
begin
 PathToDlls := IncludeTrailingPathDelimiter(PathToDlls);
 fFilter := '|Standardformate (*.mp3;*.mp2;*.mp1;*.ogg;*.wav;*.aif)'
            + '|*.mp3;*.mp2;*.mp1;*.ogg;*.wav*;*.aif';
 dlllist := TStringList.Create;
 dlllist := FindAllFiles(PathToDlls, 'bass*.dll', true);
 for ii := 0 to dlllist.Count-1 do
 begin
  Plug := BASS_PluginLoad(PAnsiChar(dlllist.Strings[ii]), 0);
   if Plug <> 0 then
   begin
    // Plugin-Info ermitteln
    Info := BASS_PluginGetInfo(Plug);
    for a := 0 to Info^.formatc - 1 do
    begin
     // Filter erweitern
     fFilter := fFilter + '|' + Info^.Formats^[a].name + ' ('
                + Info^.Formats^[a].exts + ')|' + Info^.Formats^[a].exts;
     //einzelne Erweiterungen ermitteln
     tmpext := Explode(';',Info^.Formats^[a].exts);
     for i := 0 to tmpext.Count - 1 do
      fValidExtensions.Add(StringReplace(tmpext.Strings[i],'*', '',[]));
     FreeAndNil(tmpext);
     end;
   end;
 end;
end;

function TBassPlayer.IsPlayableFile(aFilename:string):boolean;
begin
  result := fValidExtensions.IndexOf(ExtractFileExt(aFilename)) >= 0;
end;

function TBassPlayer.GetChannelLevel:Cardinal;
begin
 Result := BASS_ChannelGetLevel(fMainStream);
end;

procedure TBassPlayer.SetPan(Value:float);
begin
 BASS_ChannelSetAttribute(fMainStream, BASS_ATTRIB_PAN, Value);
end;

procedure TBassPlayer.Loop(bol:boolean);
begin
 if bol then
  BASS_ChannelFlags(fMainStream, BASS_SAMPLE_LOOP, BASS_SAMPLE_LOOP)
 else
  BASS_ChannelFlags(fMainStream, 0, BASS_SAMPLE_LOOP);
end;

end.

