unit playlist;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, bassplayer, audiofile, Contnrs;

type TPlaylistAddItemEvent =
      procedure(Sender:TObject; NewAudioFile: TAudioFile) of object;

     TPlayList = class(TBassPlayer)
      private
       fPlayingIndex:Integer;
       fPlayingFile :TAudioFile;
       fPlaylist    :TObjectList;
       fOnClear     :TNotifyEvent;
       fOnAddItem   :TPlaylistAddItemEvent;

       function GetNextAudioFileIndex:integer;
       function GetPrevAudioFileIndex:integer;
       procedure LoadFromM3U(aFilename: string);
       procedure SaveAsM3U(aFilename: string);

      public
       property PlayingFile :TAudioFile            read fPlayingFile;
       property PlayingIndex:integer               read fPlayingIndex;
       property OnClear     :TNotifyEvent          read fOnClear   write fOnClear;
       property OnAddItem   :TPlaylistAddItemEvent read fOnAddItem write fOnAddItem;

       constructor Create;
       destructor  Destroy; override;
       procedure Add(aAudioFileName:string);
       procedure Play(aIndex:integer; slide:boolean); overload;
       procedure PlayNext(slide:boolean);
       procedure PlayPrevious;
       procedure RemoveTitel(i:integer);
       procedure Clear;
       procedure LoadFromFile(aFilename: string);
       procedure SaveToFile(aFilename: string);
       function  GetTitelInfos(i:integer):TAudioFile;
       procedure ChangeTitelPos(i:integer;ort:string);
      end;

var bp:TBassPlayer;

implementation

constructor TPlaylist.Create;
begin
  inherited create;
  fPlaylist := TObjectList.Create;
  fPlayingFile := Nil;
end;


destructor TPlaylist.Destroy;
begin
  fPlaylist.Free;
  inherited Destroy;
end;

function TPlaylist.GetNextAudioFileIndex: Integer;
begin
  if fPlayingIndex = fPlaylist.Count - 1 then
    result := 0
  else
    result := fPlayingIndex + 1;
end;

function TPlaylist.GetPrevAudioFileIndex: Integer;
begin
  if fPlayingIndex = 0 then
    result := fPlaylist.Count - 1
  else
    result := fPlayingIndex - 1;
end;

procedure TPlaylist.Add(aAudioFileName:string);
var NewFile:TAudioFile;
begin
 NewFile := TAudioFile.Create;
 NewFile.GetAudioInfo(aAudioFileName);
 fPlaylist.Add(NewFile);
 if assigned(fOnAddItem) then
  fOnAddItem(Self, NewFile);
end;

procedure TPlaylist.Play(aIndex:integer; slide:boolean);
begin
 if aIndex >= 0 then
  fPlayingIndex := aIndex;
 if (fPlayingIndex > fPlaylist.Count -1) and (fPlaylist.Count > 0) then
  fPlayingIndex := fPlaylist.Count -1;
 if (fPlayingIndex >= 0) and (fPlayingIndex < fPlaylist.Count) then
 begin
  fPlayingFile := TAudioFile(fPlaylist[fPlayingIndex]);
 end;
 SetPan(0);
 play(fPlayingFile, slide);
end;

procedure TPlaylist.PlayNext(slide:boolean);
begin
 Play(GetNextAudioFileIndex, slide);
 SetPan(0);
end;

procedure TPlaylist.PlayPrevious;
begin
 Play(GetPrevAudioFileIndex, false);
 SetPan(0);
end;

procedure TPlaylist.Clear;
begin
 Stop;
 fPlaylist.Clear;
 if assigned(fOnClear) then
  fOnClear(Self);
 fPlayingFile  := nil;
 fPlayingIndex := 0;
end;

procedure TPlaylist.RemoveTitel(i:integer);
begin
 if fPlaylist.Count > 0 then
  fPlaylist.Remove(fPlaylist.Items[i]);
end;

procedure TPlaylist.LoadFromM3U(aFilename:string);
var mylist: TStringlist;
  i: Integer;
  s: string;
begin
  mylist := TStringlist.Create;
  mylist.LoadFromFile(aFilename);
  if (myList.Count > 0) then
  begin
    if (myList[0] = '#EXTM3U') then //Liste ist im EXT-Format
    begin
      i := 1;
      while (i < myList.Count) do
      begin
        // Zuerst kommen ggf. die ExtInf-Daten
        s := myList[i];
        if trim(s) = '' then
          inc(i)
        else
        begin
          if (copy(s,0,7) = '#EXTINF') then // ExtInf-Zeile auslassen
            inc(i);
          Add(ExpandFilename(myList[i]));
          inc(i);
        end;
      end;
    end
    else
      // Liste ist nicht im EXT-Format - einfach nur Dateinamen
      for i := 0 to myList.Count - 1 do
      begin
        if trim(mylist[i])='' then continue;
        Add(ExpandFilename(myList[i]));
      end;
  end;
  FreeAndNil(myList);
end;

procedure TPlaylist.SaveAsM3U(aFilename: string);
var myList: tStringlist;
    i:integer;
    aAudiofile: TAudioFile;
begin
  myList := TStringList.Create;
  myList.Add('#EXTM3U');
  for i := 0 to fPlayList.Count - 1 do
  begin
    aAudiofile := fPlaylist[i] as TAudioFile;
    myList.add('#EXTINF:' + IntTostr(aAudiofile.Dauer) + ','
          + aAudioFile.Interpret + ' - ' + aAudioFile.Titel);
    myList.Add(ExtractRelativePath(aFilename, aAudioFile.Pfad ));
  end;
  myList.SaveToFile(afilename);
  FreeAndNil(myList);
end;

procedure TPlaylist.LoadFromFile(aFilename: string);
begin
  if LowerCase(ExtractFileExt(aFilename)) = '.m3u' then
    LoadFromM3U(aFilename);
end;

procedure TPlaylist.SaveToFile(aFilename: string);
begin
  if LowerCase(ExtractFileExt(aFilename)) = '.m3u' then
    SaveAsM3U(aFilename);
end;

function TPlaylist.GetTitelInfos(i:integer):TAudioFile;
begin
 Result := TAudioFile(fPlaylist[i]);
end;

procedure TPlaylist.ChangeTitelPos(i:integer; ort:string);
begin
 if ort = 'down' then if i = 0 then exit else fplaylist.Exchange(i,i-1);
 if ort = 'up' then if i = fplaylist.Count-1 then exit else fPlaylist.Exchange(i,i+1);
end;

end.

